* {
    padding: 0;
    margin: 0;
}

body {
    background-color: #000;
    color: #96a1a1;
    font-family: Helvetica, Arial, sans-serif;
}

a {
    color: #00bc7f;
    text-decoration: none;
}

a:hover {
    color: #fcd16e;
}

#app {
    display: flex;
    flex-direction: row;
    flex-wrap: nowrap;
    align-items: stretch;
    height: 100vh;
}

#task-list {
    width: 225px;
    border-right: 1px solid #333;
    padding: 20px 0px;
}

.status-indicator {
    width: 12px;
    height: 12px;
    border-radius: 6px;
    margin-right: 10px;
    display: inline-block;
    vertical-align: middle;
}

.status-active {
    background-color: #2fbd5a;
}

.status-finished {
    background-color: #ffc539;
}

.status-failed {
    background-color: #ec155b;
}

.task-item {
    display: block;
    font-size: 14px;
    border: none;
    background-color: transparent;
    color: #ccc;
    padding: 15px 20px;
    outline: none;
    width: 100%;
    white-space: nowrap;
    text-align: left;
    overflow: hidden;
    text-overflow: ellipsis;
}

.task-item:hover {
    background-color: #191919;
}

.task-view {
    flex-grow: 1;
    padding-left: 20px;
    max-width: 800px;

    display: flex;
    flex-direction: column;
    flex-wrap: nowrap;
    align-items: stretch;
}

.task-output {
    flex-grow: 1;
    background-color: black;
    padding: 10px;
    overflow: scroll;
    height: 300px;
    color: #fff;
    border-radius: 10px;
    border: 1px solid #333;
    font-family: 'San Francisco', 'Menlo', 'Monaco', monospace;
}

.task-output-disconnected {
    border-color: #d85720;
}

.task-status {
    line-height: 1.5em;
    padding: 20px 10px;
}

.task-status .status {
    color: #666;
    font-variant: small-caps;
}

.task-actions {
    padding: 20px 10px;
}

.task-actions button {
    background-color: transparent;
    color: #96a1a1;
    border: 1px solid #333;
    padding: 10px;
    font-size: 16px;
    outline: none;
    min-width: 120px;
    margin-right: 20px;
}

.task-actions button:hover {
    border-color: #00bc7f;
    color: #fff;
}
