<!DOCTYPE html>
<html lang="en-US">

<head>
    <title>{{session.name or 'Oxen'}}</title>
    <link href="/static/css/app.css" rel="stylesheet" />
</head>

<body>
    <div id="app"></div>
    <script src="/static/js/oxen.js"></script>
</body>

</html>